/* src/help.c: identify a symbol

   Copyright (c) 1989-94 James E. Wilson, Robert A. Koeneke

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Library General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "config.h"
#include "constant.h"
#include "types.h"
#include "externs.h"
#include <stdlib.h>
#include <string.h>


/* zero is rock digging, 1-6 negative values are stores, 
 * positive values are TVALs,
 * -7 is how to attack 
 * -8 is what we see after a kill.*/
int
should_display_extra_assistance (int tval)
{
  if (display_extra_assistance == FALSE)
    return FALSE;
  
  if (tval > 0)
    return object_assistance_for_new_players_seen[tval] <= SHOW_NEWBIE_TIMES-1;
  else if (tval == DIGGING_ASSIST)
    return digging_seen <= SHOW_NEWBIE_TIMES-1;
  else if (tval >= -6)
    return store_seen[(tval*-1)-1] <= SHOW_NEWBIE_TIMES-1;
  else if (tval == ATTACK_ASSIST)
    return attack_seen <= SHOW_NEWBIE_TIMES-1;
  else if (tval == KILL_ASSIST)
    return kill_seen <= SHOW_NEWBIE_TIMES-1;
  else if (tval == INITIAL_ASSIST)
    return initial_assist_seen <= SHOW_NEWBIE_TIMES-1;
  else if (tval == FIFTY_FEET_ASSIST)
    return fifty_feet_seen <= SHOW_NEWBIE_TIMES-1;
  else if (tval == CURSED_ASSIST)
    return cursed_seen <= SHOW_NEWBIE_TIMES-1;
  else if (tval == POISONED_ASSIST)
    return poison_seen <= SHOW_NEWBIE_TIMES-1;
  else if (tval == IDENTIFY_RINGS_AMULETS_ASSIST)
    return identify_rings_amulets_seen <= SHOW_NEWBIE_TIMES-1;
  else if (tval == IDENTIFY_WANDS_STAVES_ASSIST)
    return identify_wands_staves_seen <= SHOW_NEWBIE_TIMES-1;
  else if (tval == IDENTIFY_WEAPONS_ASSIST)
    return identify_weapons_seen <= SHOW_NEWBIE_TIMES-1;
  else if (tval == IDENTIFY_BOWS_ASSIST)
    return identify_bows_seen <= SHOW_NEWBIE_TIMES-1;
  else if (tval == IDENTIFY_ARMOR_ASSIST)
    return identify_armor_seen <= SHOW_NEWBIE_TIMES-1;
  else if (tval == IDENTIFY_POTIONS_ASSIST)
    return identify_potions_seen <= SHOW_NEWBIE_TIMES-1;
  else if (tval == IDENTIFY_SCROLLS_ASSIST)
    return identify_scrolls_seen <= SHOW_NEWBIE_TIMES-1;
  return FALSE;
}

int
should_display_extra_identify_assistance (int tval)
{
  return
    ((tval == TV_RING || tval == TV_AMULET) && 
     should_display_extra_assistance (IDENTIFY_RINGS_AMULETS_ASSIST)) ||

    ((tval == TV_WAND || tval == TV_STAFF) &&
     should_display_extra_assistance (IDENTIFY_WANDS_STAVES_ASSIST)) ||

    ((tval == TV_SWORD || tval == TV_POLEARM || tval == TV_HAFTED) &&
     should_display_extra_assistance (IDENTIFY_WEAPONS_ASSIST)) ||

    (tval == TV_BOW && 
     should_display_extra_assistance (IDENTIFY_BOWS_ASSIST)) ||

    ((tval == TV_SHIELD || tval == TV_HARD_ARMOR || tval == TV_SOFT_ARMOR ||
      tval == TV_GLOVES || tval == TV_BOOTS || tval == TV_HELM ||
      tval == TV_CLOAK)) &&
    should_display_extra_assistance (IDENTIFY_ARMOR_ASSIST) ||

    ((tval == TV_POTION1 || tval == TV_POTION2) &&
     should_display_extra_assistance (IDENTIFY_POTIONS_ASSIST)) ||

    (tval == TV_SCROLL &&
     should_display_extra_assistance (IDENTIFY_SCROLLS_ASSIST));
}

/* zero is rock digging, negative values are stores, 
 * positive values are TVALs */
void
increment_extra_assistance (int tval)
{
  if (display_extra_assistance == FALSE)
    return;
  if (tval > 0)
    object_assistance_for_new_players_seen[tval]++;
  else if (tval == DIGGING_ASSIST)
    digging_seen += SHOW_NEWBIE_TIMES;
  else if (tval >= -6)
    store_seen[(tval*-1)-1]++;
  else if (tval == ATTACK_ASSIST)
    attack_seen++;
  else if (tval == KILL_ASSIST)
    kill_seen += SHOW_NEWBIE_TIMES;
  else if (tval == INITIAL_ASSIST)
    initial_assist_seen += SHOW_NEWBIE_TIMES;
  else if (tval == FIFTY_FEET_ASSIST)
    fifty_feet_seen += SHOW_NEWBIE_TIMES;
  else if (tval == CURSED_ASSIST)
    cursed_seen += SHOW_NEWBIE_TIMES;
  else if (tval == POISONED_ASSIST)
    poison_seen += SHOW_NEWBIE_TIMES;
  else if (tval == IDENTIFY_RINGS_AMULETS_ASSIST)
    identify_rings_amulets_seen += SHOW_NEWBIE_TIMES;
  else if (tval == IDENTIFY_WANDS_STAVES_ASSIST)
    identify_wands_staves_seen += SHOW_NEWBIE_TIMES;
  else if (tval == IDENTIFY_WEAPONS_ASSIST)
    identify_weapons_seen += SHOW_NEWBIE_TIMES;
  else if (tval == IDENTIFY_BOWS_ASSIST)
    identify_bows_seen += SHOW_NEWBIE_TIMES;
  else if (tval == IDENTIFY_ARMOR_ASSIST)
    identify_armor_seen += SHOW_NEWBIE_TIMES;
  else if (tval == IDENTIFY_POTIONS_ASSIST)
    identify_potions_seen += SHOW_NEWBIE_TIMES;
  else if (tval == IDENTIFY_SCROLLS_ASSIST)
    identify_scrolls_seen += SHOW_NEWBIE_TIMES;
}

char *
get_extra_assistance (int tval)
{
  char *msg = NULL;
  if (should_display_extra_assistance (tval))
    {
      increment_extra_assistance (tval);
      /* potions have two TVALs so we treat them equally */
      if (tval == TV_POTION1)
        increment_extra_assistance (TV_POTION2);
      else if (tval == TV_POTION2)
        increment_extra_assistance (TV_POTION1);
      switch (tval)
        {
        case TV_NOTHING:
          msg = "It can't be used in any way.";
          break;
        case TV_MISC:
          msg = "It has no use at all.  The dungeon has useless things scattered in it, and this is one of those things.  You can pick it up, and maybe throw it, but that's about it.";
          break;
        case TV_CHEST:
          msg = rogue_like_commands ?
            "It can be (o)pened to find treasure inside.  (s)earching around a chest can sometimes detect a trap, and it can be subsequently (D)isarmed.  Experience points are gained when disarming trapped chests.":
            "It can be (o)pened to find treasure inside.  (S)earching around a chest can sometimes detect a trap, and it can be subsequently (d)isarmed.  Experience points are gained when disarming trapped chests";
          break;
        case TV_SLING_AMMO:
          msg = rogue_like_commands ?
            "It is for (t)hrowing at enemies while (w)ielding a sling.":
            "It is for (f)iring at enemies while (w)ielding a sling.";
          break;
        case TV_BOLT:
          msg = rogue_like_commands ?
            "It is for (t)hrowing at enemies when already having (w)ielded a crossbow.":
            "It is for (f)iring at enemies when already having (w)ielded a crossbow.";
          break;
        case TV_ARROW:
          msg = rogue_like_commands ?
            "It is for (t)hrowing at enemies while (w)ielding a bow.":
            "It is for (f)iring at enemies while (w)ielding a bow.";
          break;
        case TV_SPIKE:
          msg = rogue_like_commands ? "It is for (S)piking into closed doors when fleeing enemies.":
            "It is for (j)amming into closed doors when fleeing enemies.";
          break;
        case TV_LIGHT:
          msg =  "It is for illuminating the dungeon.  To turn it on, you have to (w)ear it.  Without light you can't see where you're going, and then what will you do?";
          break;
        case TV_BOW:
          msg = rogue_like_commands ? 
            "It is for (t)hrowing arrows at your enemies after you've already (w)ielded it.":
            "It is for (f)iring arrows at your enemies after you've already (w)ielded it.";
          break;
        case TV_HAFTED:
        case TV_SWORD:
        case TV_POLEARM:
          msg =  "It is for attacking your enemies.  You attack by moving into them.";
          break;
        case TV_DIGGING:
          msg = rogue_like_commands ? 
            "It is for digging into rock.  To start tunneling you first have to (w)ield it, then hold down CTRL while simultaneously pressing a direction key.  Mining precious metals is a valuable source of income for new adventurers.":
            "It is for (T)unneling into rock after you have (w)ielded it.  Mining precious metals is a valuable source of income for new adventurers.";
          break;
        case TV_BOOTS:
          msg =  "They protect your feet from your foes.  You can (w)ear these.";
          break;
        case TV_GLOVES:
          msg = "They protect your hands from the attacks of your enemies.  You can (w)ear these.";
          break;
        case TV_CLOAK:
          msg = "It is (w)orn on top of your armor and usually provides minimal protection.";
          break;
        case TV_HELM:
          msg = "It is for protecting your cranium against the attacks of your enemies.  You can (w)ear it.";
          break;
        case TV_SHIELD:
          msg = "It is for protecting your upper body against the attacks of your enemies.  You can (w)ear it.";
          break;
        case TV_HARD_ARMOR:
        case TV_SOFT_ARMOR:
          msg = "It is for protecting your body against the attacks of your enemies.  You can wear it.";
          break;
        case TV_AMULET:
          msg = "It is for (w)earing around your neck.  Many of these are magical in nature.";
          break;
        case TV_RING:
          msg = "It is for (w)earing on your fingers.  Many of these are magical in nature.";
          break;
        case TV_STAFF:
          msg = rogue_like_commands ?
            "It is for casting magical spells.  (Z)apping a staff can do a variety of things, like attack an enemy, or heal you, or heal your enemies, or tell you something about the dungeon.  When it runs out of charges, it can be recharged using a scroll or spell.  Experience points are gained by zapping a new staff.":
            "It is for casting magical spells.  (u)sing a staff can do a variety of things, like attack an enemy, or heal you, or heal your enemies, or tell you something about the dungeon.  When it runs out of charges, it can be recharged using a scroll or spell.  Experience points are gained by using a new staff.";
          break;
        case TV_WAND:
          msg = rogue_like_commands ?
            "It is for attacking your enemies in a given direction.  (z)apping a wand at your enemies is usually a good idea.  When it runs out of charges, it can be recharged using a scroll or spell.  Experience points are gained by using a new wand.":
            "It is for attacking your enemies in a given direction.  (a)iming and firing a wand at your enemies is usually a good idea.  When it runs out of charges, it can be recharged using a scroll or spell.  Experience points are gained by using a new wand.";
          break;
        case TV_SCROLL:
          msg = "It is for casting magical spells.  (r)eading a scroll can do a wide variety of things; some will help you, while others will help your enemies.  To read a scroll, you'll need some light to read by.  A scroll can only be read once, and then the parchment self-combusts.  Experience points are gained by reading a new scroll  Experience points are gained by reading a new scroll.";
          break;
        case TV_POTION1:
        case TV_POTION2:
          msg = "It is for (q)uaffing (drinking).  Quaffing a potion can have a variety of effects, sometimes good and sometimes bad.  Experience points are gained by quaffing new potions.";
          break;
        case TV_FLASK:
          msg = rogue_like_commands ?
            "It is for (F)illing a lantern.  Lanterns use this as fuel.  In a pinch you can (t)hrow this as a molotov cocktail at an enemy.":
            "It is for (F)illing a lantern.  Lanterns use this as fuel.  In a pinch you can (f)ire this as a molotov cocktail at an enemy.";
          break;
        case TV_FOOD:
          msg = "It is for (E)ating.  Yum.  If you don't eat, you'll starve -- then who will kill the Balrog?  Experience points are gained by eating new foods.  Eating mushrooms or hairy molds can have magical effects.";
          break;
        case TV_MAGIC_BOOK:
          msg = rogue_like_commands ?
            "It is for (m)ages, rogues, and rangers to cast spells from.  This book can be (P)erused, and periodically you can (G)ain a new spell by studying it.  Experience points are gained by casting new spells.":
            "It is for (m)ages, rogues, and rangers to cast spells from.  This book can be (b)rowsed, and periodically you can (G)ain a new spell by studying it.  Experience points are gained by casting new spells.";
          break;
        case TV_PRAYER_BOOK:
          msg = rogue_like_commands ?
            "It is for priests and paladins to (p)ray from.  This book can be (P)erused, and periodically a random new prayer will be learned.  Experience points are gained by using new prayers.":
            "It is for priests and paladins to (p)ray from.  This book can be (b)rowsed, and periodically a random new prayer will be learned.  Experience points are gained by using new prayers.";
          break;
        case TV_GOLD:
          msg = "It is for buying goods in the stores.";
          break;
        case TV_INVIS_TRAP:
        case TV_VIS_TRAP:
          msg = "It hurts you when you step on it, unless it was a booby trap and has already gone off.  You can (D)isarm traps, but sometimes this inadvertantly sets them off.  Experience points can be gained by disarming traps.  Traps are usually invisible, and you can use scrolls, staves and spells to detect them.";
          break;
        case TV_RUBBLE:
          msg = "It is an obstacle in the dungeon that can be removed by tunneling it out.  Sometimes there is an object underneath the rubble.";
          break;
        case TV_OPEN_DOOR:
          msg = "(c)losing a door can sometimes prevent your enemies from chasing you.  Experience points are gained by lockpicking doors.";
          break;
        case TV_CLOSED_DOOR:
          msg = rogue_like_commands ?
            "(o)pening a door can help you flee from your enemies.  If you have a (S)pike you can make it unopenable.  You can also (f) a door open.":
            "(o)pening a door can help you flee from your enemies.  If you have a spike, you can (j)am it to make it unopenable.  You can also (B)ash a door open.";
          break;
        case TV_UP_STAIR:
          if (dun_level == 1)
            msg = "It takes you up to the town when you stand on it and press the (<) key.  Enemies won't follow you up the stairs.";
          else
            msg = "It takes you higher in the dungeon when you stand on it and press the (<) key.  Enemies won't follow you up the stairs.";
          break;
        case TV_DOWN_STAIR:
          if (dun_level == 0)
            msg = "It takes you into the dungeon when you stand on it and press the (>) key.  Enemies won't follow you down the stairs.";
          else
            msg = "It takes you deeper in the dungeon when you stand on it and press the (<) key.  Enemies won't follow you down the stairs.";
          break;
        case TV_SECRET_DOOR:
          msg = "It's something you can find when searching the dungeon.";
          break;
        case TV_STORE_DOOR:
          msg = "To visit a store just step on it.";
          break;
        default:
          msg = "error in get_extra_assistance";
          break;
        }
    }
  if (msg)
    return strdup (msg);
  return msg;
}

void
display_fifty_feet_assistance ()
{
  roff_msg_start ();
  roff_msg_append ("Welcome to 50 feet, the first dungeon depth in the mines of Moria.\n\n");
  roff_msg_append ("Although you just came down some stairs, there was a one-way door that snapped shut as you came through.  To go back up to the town you will have to find some up stairs (the '<' symbol).");
  if (inventory[INVEN_LIGHT].tval == TV_NOTHING &&
      inventory[INVEN_WIELD].tval == TV_NOTHING)
    roff_msg_append ("\n\nYou really need to (w)ear a light source and (w)ield a weapon.");
  else if (inventory[INVEN_LIGHT].tval == TV_NOTHING)
    roff_msg_append ("\n\nYou really need to (w)ear a light source.");
  else if (inventory[INVEN_WIELD].tval == TV_NOTHING)
    roff_msg_append ("\n\nYou really need to (w)ield a weapon.");
  if (py.flags.new_spells)
    roff_msg_append ("  Don't forget that you can (G)ain some magic spells.");
  roff_msg_append ("\n\nGood luck on your quest to slay the Balrog at the bottom of the dungeon!");
  roff_msg_end (-1, 0, 0);

  increment_extra_assistance (FIFTY_FEET_ASSIST);
}

void
display_initial_assistance ()
{

  roff_msg_start ();
  roff_msg_append ("Welcome to the game!\n\n");
  roff_msg_append (PACKAGE);
  roff_msg_append (" has two different keyboard command-sets, the Moria command-set which uses the keypad for movement, and the Rogue command-set which does not.\n\n");
  roff_msg_append ("Do you want to use the Rogue command set? (y/n)\n");
  int query = roff_msg_end (TRUE, 0, 0);
  if (query == 'Y' || query == 'y')
    {
      rogue_like_commands = TRUE;
      keybinding = KEYBINDING_ROGUELIKE;
    }
  roff_msg_start ();
  if (rogue_like_commands == FALSE)
    roff_msg_append ("Great!  You are the @ symbol on the screen, and you can use the keypad to move your player around the town.  2 moves north, 5 moves east, and so on.");
  else
    roff_msg_append ("Great!  You are the @ symbol on the screen, and you can use these keys for movement:\n\
                     y  k  u\n\
                     h  .  l  [. is stay]\n\
                     b  j  n");
  roff_msg_append ("\n\nThe first thing you should do is (w)ear the items in your pack, (e.g. press the 'w' key to wear things) and then visit the stores to buy some weapons and armor before going down into the dungeon.");
  if (py.flags.new_spells)
    roff_msg_append ("  You can also (G)ain some magic spells.");
  roff_msg_append ("\n\nOnce you're ready to go into the dungeon, walk on top of the stairs (the '>' symbol)  and then press the '>' key to go down.");

  if (rogue_like_commands == FALSE)
    roff_msg_append ("  Lastly, if you're new to all this, it helps if you (l)ook around.");
  else
    roff_msg_append ("  Lastly, if you're new to all this, it helps if you e(x)amine your surroundings.");
  roff_msg_append ("\n\nGood Luck!");
  roff_msg_end (-1, 0, 0);
  increment_extra_assistance (INITIAL_ASSIST);
  prt ("Type '?' for help.", 0, 0);
}

void
display_store_assistance (char tchar)
{
  switch (tchar)
    {
    case '1':
      roff_msg_append ("  This is where adventurers can buy the essentials for exploring the mines of Moria, such as digging tools, food and torches.");
      break;
    case '2':
      roff_msg_append ("  This is where adventurers can buy and sell shields, boots, body armor and more.");
      break;
    case '3':
      roff_msg_append ("  This is where adventurers can buy and sell swords, polearms, daggers, hafted weapons, as well as bows and crossbows.");
      break;
    case '4':
      roff_msg_append ("  This is where adventurers can buy and sell the priestly books, potions, scrolls, and sometimes weapons.");
      break;
    case '5':
      roff_msg_append ("  This is where adventurers can buy and sell all manner of useful scrolls and potions.");
      break;
    case '6':
      roff_msg_append ("This is where adventurers can buy and sell spell-books, wands, staves, and sometimes rings and amulets.");
      break;
    default:
      break;
    }
  increment_extra_assistance ('0' - tchar);
}

void
display_digging_assistance ()
{
  if  (rogue_like_commands)
    roff_msg_append ("  You can dig into rock by (w)ielding a digging tool, and then hold down CTRL while simultaneously pressing a direction key.  Mining precious metals is a valuable source of income for new adventurers.");
  else
    roff_msg_append ("  You can dig into rock by (w)ielding a digging tool, and then (T)unneling in a given direction.  Mining precious metals is a valuable source of income for new adventurers.");
  increment_extra_assistance (DIGGING_ASSIST);
}

void
display_poisoned_assistance ()
{
  roff_msg_start ();
  roff_msg_append ("You have been poisoned.  ");
  roff_msg_append ("Your hit-points will continue to drop while the poison is still active in your system.  ");
  roff_msg_append ("Sometimes it causes death, and other times the poison simply runs it course and leaves you weakened.\n\n");
  roff_msg_append ("There are many ways to address being poisoned.  If a monster poisoned you, get away from it immediately!  Quaffing a potion might neutralize the poison (or hurt you more), and other potions are known to simply slow the poison so that your hitpoints stop dropping so fast.  The townsfolk say that certain mushrooms will cure poison when eaten.");
  if (class[py.misc.pclass].spell == MAGE)
    roff_msg_append ("  Also, you might be able to cast the Cure Poison spell.");
  else if (class[py.misc.pclass].spell == PRIEST)
    roff_msg_append ("  Also, you might be able to pray the Slow Poison, or Neutralize Poison prayers.");
  roff_msg_append ("  In the worst-case scenario you can try making your way to the Temple in the town, and hope that the poison runs its course.");
  roff_msg_end (-1, 0, 0);
  increment_extra_assistance (POISONED_ASSIST);
}

void
display_cursed_assistance (int tval, int slot)
{
  roff_msg_start ();
  char *verb = "worn";
  char *location = NULL;
  if (weaponsmith (tval))
    verb = "wielded";
  vtype msg;
  snprintf (msg, sizeof (msg), "You have %s a cursed item.  ", verb);
  roff_msg_append (msg);

  switch (slot)
    {
    case INVEN_WIELD:
    case INVEN_HANDS: location = "hands"; break;
    case INVEN_HEAD: location = "head"; break;
    case INVEN_NECK: location = "neck"; break;
    case INVEN_BODY: location = "body"; break;
    case INVEN_ARM: location = "arm"; break;
    case INVEN_RIGHT: 
    case INVEN_LEFT: location = "finger"; break;
    case INVEN_FEET: location = "feet"; break;
    case INVEN_OUTER: location = "shoulders"; break;
    default: break;
    }
  if (location)
    {
      if (strcmp (verb, "wielded") == 0)
        snprintf (msg, sizeof (msg), "You will not be able to take it out of your %s because it is a magically damned item.  ", location);
      else
        snprintf (msg, sizeof (msg), "You will not be able to take it off your %s because it is a magically damned item.  ", location);
      roff_msg_append (msg);
    }
  roff_msg_append ("You should proceed to the Temple in the town and buy a Scroll of Remove Curse, so that you can remove it.");
  if (class[py.misc.pclass].spell == MAGE)
    {
      roff_msg_append ("  At some point in the future you might be able to cast the Remove Curse spell yourself.");
    }
  else if (class[py.misc.pclass].spell == PRIEST)
    {
      roff_msg_append ("  At some point in the future you might be able to pray the Remove Curse prayer yourself.");
    }
  roff_msg_end (-1, 0, 0);
  increment_extra_assistance (CURSED_ASSIST);
}

void
display_pickup_assistance (int tval, char *object_name, int slot)
{
  char *explainer = get_extra_assistance (tval);
  if (explainer)
    {
      int i;
      bigvtype msg;
      roff_msg_start ();
      char *tmp = strrchr (object_name, '.');
      if (tmp)
        *tmp = ',';
      snprintf (msg, sizeof (msg), "You have picked up %s "
                "and put it inside your backpack at slot (%c).  ",
                object_name, slot + 'a');
      if (tmp)
        *tmp = '.';
      roff_msg_append (msg);
      roff_msg_append (explainer);
      roff_msg_end (-1, 0, 0);
      for (i = 0; i < SHOW_NEWBIE_TIMES-1; i++)
        increment_extra_assistance (tval);
      free (explainer);
    }
}

static void
roff_msg_two_random_materials (char **array, int max)
{
  if (max > 1)
    {
      int material1 = randint (max) - 1, material2;
      vtype msg;
      do
        {
          material2 = randint (max) - 1;
        } while (material1 != material2);
      snprintf (msg, sizeof (msg), "%s, and %s", array[material1], array[material2]);
      roff_msg_append (msg);
    }
}

static void
roff_msg_identify_notice (char *s1)
{
  bigvtype msg;
  snprintf (msg, sizeof (msg), "To find out the true nature of this %s, it needs to be identified, which can be done with a Scroll of Identify.", s1);
  roff_msg_append (msg);
  if (class[py.misc.pclass].spell == MAGE)
    roff_msg_append ("  Or if you have sufficent spellcasting ability, you can cast the Identify spell.\n\n");
  else
    roff_msg_append ("\n\n");
}

static void
roff_msg_colored_not_fully_identified (char *s1, char *s2, char **array, int max, char *example1, char *example2)
{
  bigvtype msg;
  snprintf (msg, sizeof (msg), "The %s you picked up isn't fully identified yet, which is why it is named after what it looks like, rather than what it does.  ", s1);
  roff_msg_append (msg);
  if (max > 1)
    {
      roff_msg_append (s2);
      roff_msg_two_random_materials (array, max);
      roff_msg_append (".  ");
    }
  else
    roff_msg_append ("  ");
  snprintf (msg, sizeof (msg), "Once it is fully identified it will say something like '%s', or '%s' instead.  ", example1, example2);

  roff_msg_append (msg);
  roff_msg_identify_notice (s1);
}

void
display_identify_assistance (inven_type *i_ptr)
{
  bigvtype msg;
  int tval = i_ptr->tval;
  roff_msg_start ();
  char *item = NULL;
  int wearable = 0, wieldable = 0, usable = 0;
  vtype tmp_str;
  objdes (tmp_str, sizeof (tmp_str), i_ptr, TRUE);
  
  roff_msg_append ("You have ");
  roff_msg_append (tmp_str);
  roff_msg_append ("\n\n");
  if (tval == TV_RING)
    {
      item = "ring";
      wearable = 1;
      roff_msg_colored_not_fully_identified (item, "Rings are made of materials like ", rocks, MAX_ROCKS, "Ring of Strength", "Ring of Weakness");
      increment_extra_assistance (IDENTIFY_RINGS_AMULETS_ASSIST);
    }
  else if (tval == TV_AMULET)
    {
      item = "amulet";
      wearable = 1;
      roff_msg_colored_not_fully_identified (item, "Amulets are made of materials like ", amulets, MAX_AMULETS, "Amulet of Resist Acid", "Amulet of Slow Digestion");
      increment_extra_assistance (IDENTIFY_RINGS_AMULETS_ASSIST);
    }
  else if (tval == TV_WAND)
    {
      item = "wand";
      usable = 1;
      roff_msg_colored_not_fully_identified (item, "Wands are made of different kinds of metals like ", metals, MAX_METALS, "Wand of Magic Missile (10 charges)", "Wand of Haste Monster (6 charges)");
      increment_extra_assistance (IDENTIFY_WANDS_STAVES_ASSIST);
    }
  else if (tval == TV_STAFF)
    {
      item = "staff";
      usable = 1;
      roff_msg_colored_not_fully_identified (item, "Staves are made from different kinds of wood like ", woods, MAX_WOODS, "Staff of Light (13 charges)", "Staff of Earthquakes (6 charges)");
      increment_extra_assistance (IDENTIFY_WANDS_STAVES_ASSIST);
    }
  else if (tval == TV_SWORD || tval == TV_POLEARM || tval == TV_HAFTED)
    {
      wieldable = 1;
      item = "weapon";
      roff_msg_append ("This weapon name appears a different than the similarly named items in the stores because it is unidentified.  ");
      roff_msg_identify_notice (item);
      roff_msg_append ("Once it is fully identified, it will say something like 'a Halberd (3d4) (+4,+5)' or 'a War Hammer (3d3) (+1,+2)'.  The (3d4) portion is the hitdice of the weapon.  The (+4,+5) portion is a chance-to-hit bonus and a damage bonus.\n\n");
      roff_msg_end (-1, 0, 0);
      roff_msg_start ();
      roff_msg_append ("Sometimes a weapon can have a special magical ability.  There is (HA) for Holy Avenger, (DF) for Defender, (FT) for Flame Tongue, (FB) for Frost Brand, (SD) for Slay Dragon, (SA) for Slay Animal, (SE) for Slay Evil, and (SU) for Slay Undead.\n\n");
      roff_msg_append ("The Holy Avenger and the Defender are the most rare and best weapons, and they confer all sorts of fantastic abilities on the wielder.  You would be very lucky to find one of these!\n\n");
      increment_extra_assistance (IDENTIFY_WEAPONS_ASSIST);
    }
  else if (tval == TV_BOW)
    {
      item = "projectile launcher";
      wearable = 1;
      roff_msg_append ("This item name appears a little different than the similarly named items in the stores because it is unidentified.  ");
      roff_msg_identify_notice (item);
      roff_msg_append ("Once it is fully identified, it will say something like 'a Composite Bow (x2) (+0,+1)' or 'a Sling (x3) (+1,+2)'.  The (x2) portion is a damage multiplier.  The (+0,+1) portion is a chance-to-hit bonus and a damage bonus.\n\n");
      increment_extra_assistance (IDENTIFY_BOWS_ASSIST);
    }
  else if (tval == TV_SHIELD || tval == TV_HARD_ARMOR || tval == TV_SOFT_ARMOR 
           || tval == TV_GLOVES || tval == TV_BOOTS || tval == TV_HELM ||
           tval == TV_CLOAK)
    {
      item = "armor";
      wearable = 1;
      roff_msg_append ("This armor name appears different than the similarly named items found in the stores because it is unidentified.  ");
      roff_msg_identify_notice (item);
      roff_msg_append ("Once it is identified, it will say something like 'Hard Leather Ring Mail (-2) [8,+3].' or 'a Cloak [1,+4].'.  When you see a '(-2)' it is displaying a penalty on the chance-to-hit an enemy while wearing this armor.  When you see '[8,+3]' it means that the armor has a base armor class of 8, and a bonus of 3, for a total of 11 armor class while wearing it.  The bonus portion can be modified by Scrolls of Enchant Armor.");
      roff_msg_end (-1, 0, 0);
      roff_msg_start ();
      roff_msg_append ("Sometimes shields and body armor will have (RA) in the name, this means it confers a resistance to acid attacks on the wearer.  There is also (RF) for resist fire, (RC) for resist cold, (RL) for resist lightning, and (R) for resistance to acid, fire, cold and lightning.\n\n");
      roff_msg_append ("Gloves, boots, helms and cloaks will sometimes have a special magical ability like: 'Cloak of Protection (+2)', or 'Boots of Speed (+1)', but it is very rare.  The magical abilities vary for each kind of armor, and the (+1) indicates the degree of effect.\n\n");

      increment_extra_assistance (IDENTIFY_ARMOR_ASSIST);
    }
  else if (tval == TV_POTION1 || tval == TV_POTION2)
    {
      item = "potion";
      roff_msg_colored_not_fully_identified (item, "Potions come in many colors and dispositions like ", colors, MAX_COLORS, "Potion of Intelligence", "Potion of Poison");
      roff_msg_append ("You can try quaffing the potion, which will identify it if you happen to notice its effect.  Take care though, because it could be an unhealthy potion to drink!\n\n");
      increment_extra_assistance (IDENTIFY_POTIONS_ASSIST);
    }
  else if (tval == TV_SCROLL)
    {
      item = "scroll";
      roff_msg_append ("The scroll you picked up isn't identified yet, so all we can go by are the strange letters written on the top of it.  ");
      roff_msg_identify_notice (item);
      roff_msg_append ("Once it is fully identified it will say something like 'Scroll of Light' or 'Scroll of Phase Door' instead.  ");
      roff_msg_append ("You can try reading the scroll, which will identify it if you happen to notice its effect.  Beware reading unidentified scrolls, because it could be spell that is very hazardous to your health!\n\n");
      increment_extra_assistance (IDENTIFY_SCROLLS_ASSIST);
    }
      
  if (wearable || wieldable)
    {
      snprintf (msg, sizeof (msg), "You can always try %s the %s without identifying it, but it could be cursed.  ", wearable ? "wearing" : "wielding", item);
      roff_msg_append (msg);
    }
  if (usable)
    {
      snprintf (msg, sizeof (msg), "You can try using the %s, which could identify it if you notice its effect.  Identifying things in this way means you won't know how many charges it has left.  ");
      roff_msg_append (msg);
    }

  snprintf (msg, sizeof (msg), "Selling the %s in a store will identify it, but then you won't own it anymore, and buying it back could be very expensive if it's particularly useful.", item);
  roff_msg_append (msg);

  roff_msg_end (-1, 0, 0);
}

void
display_object_quantity_and_weight (inven_type *i_ptr)
{
  /* show the quantity and weight */
  int total_weight = i_ptr->weight * i_ptr->number;
  vtype msg;
  if (i_ptr->number == 1)
    snprintf (msg, sizeof (msg), "It weighs %d.%d pounds",
              (total_weight) / 10, (total_weight) % 10);
  else
    snprintf (msg, sizeof (msg), 
              "You have %d of them, and they weigh a total of %d.%d pounds",
              i_ptr->number, (total_weight) / 10, (total_weight) % 10);
  roff_msg_append (msg);
}

static char *
get_item_kind_name (int tval)
{
  char *name = NULL;
  switch (tval)
    {
    case TV_MISC: name = "miscellaneous item"; break;
    case TV_CHEST: name = "chest"; break;
    case TV_SLING_AMMO: name = "sling ammunition"; break;
    case TV_BOLT: name = "bolt"; break;
    case TV_ARROW: name = "arrow"; break;
    case TV_SPIKE: name = "spike"; break;
    case TV_LIGHT: name = "light source"; break;
    case TV_BOW: name = "projectile launcher"; break;
    case TV_HAFTED: 
    case TV_POLEARM:
    case TV_SWORD: name = "weapon"; break;
    case TV_DIGGING: name = "digging tool"; break;
    case TV_BOOTS: name = "pair of boots"; break;
    case TV_GLOVES: name = "pair of gloves"; break;
    case TV_CLOAK: name = "cloak"; break;
    case TV_SHIELD: name = "shield"; break;
    case TV_HELM: name = "headwear"; break;
    case TV_HARD_ARMOR:
    case TV_SOFT_ARMOR: name = "body armor"; break;
    case TV_AMULET: name = "amulet"; break;
    case TV_RING: name = "ring"; break;
    case TV_STAFF: name = "staff"; break;
    case TV_WAND: name = "wand"; break;
    case TV_SCROLL: name = "scroll"; break;
    case TV_POTION1:
    case TV_POTION2: name = "potion"; break;
    case TV_FLASK: name = "flask"; break;
    case TV_FOOD: name = "foodstuff"; break;
    case TV_MAGIC_BOOK: 
    case TV_PRAYER_BOOK: name = "book"; break;
    default:
      break;
    }
  if (name)
    return strdup (name);
  return NULL;
}

void
roff_msg_charges_left (int num)
{
  if (num == 0)
    roff_msg_append ("no charges left");
  else if (num == 1)
    roff_msg_append ("a single charge left");
  else
    roff_msg_printf ("%d charges left", num);
}

static void
roff_msg_points (int num)
{
  if (num == 0)
    roff_msg_append ("0 points");
  else if (num == 1)
    roff_msg_append ("1 points");
  else 
    roff_msg_printf ("%d points", num);
}

void
display_ego_abilities (inven_type *i_ptr, char *s)
{
  if (!known2_p (i_ptr))
    {
      if (i_ptr->ident & ID_MAGIK)
        roff_msg_printf ("%sIt appears to be magically enhanced somehow.", s);
      else if (i_ptr->ident & ID_DAMD)
        roff_msg_printf ("%sIt is damned.", s);
      else if (i_ptr->ident & ID_EMPTY)
        roff_msg_printf ("%sIt is empty.", s);
      return;
    }
  char *name = get_item_kind_name (i_ptr->tval);
  if (!name)
    return;
  if (i_ptr->name2)
    {
      roff_msg_printf ("%sThis ", s);
      roff_msg_append (name);
    }
  switch (i_ptr->name2)
    {
    case SN_R:
      roff_msg_append (" magically makes the wearer resistant to many kinds of attacks.  It provides resistance to fire, cold, lightning and acid attacks.  Only one third of the regular damage is done.  (R) stands for Resistance.");
      break;
    case SN_RA:
      roff_msg_append (" magically makes the wearer resistant to acid attacks.  Only one third of the regular damage is done.  (RA) stands for Resist Acid.");
      break;
    case SN_RF:
      roff_msg_append (" magically makes the wearer resistant to flame-based attacks.  Only one third of the regular damage is done.  (RF) stands for Resist Fire.");
      break;
    case SN_RC:
      roff_msg_append (" magically makes the wearer resistant to cold and frost-based attacks.  Only one third of the regular damage is done.  (RC) stands for Resist Cold.");
      break;
    case SN_RL:
      roff_msg_append (" magically makes the wearer resistant to electrical or lightning-based attacks.  Only one third of the regular damage is done.  (RL) stands for Resist Lightning.");
      break;
    case SN_HA:
      roff_msg_append (" is a very fine magical weapon that makes the wielder able to see invisible creatures, does double damage against evil enemies, and triple damage against the undead.  The wielder's strength is increased by ");
      roff_msg_points (i_ptr->p1);
      roff_msg_append (", and it gives an armor class bonus of ");
      roff_msg_points (i_ptr->toac);
      roff_msg_append (".  Also the wielder's ");
      switch (i_ptr->p1)
        {
        case 1:
          roff_msg_append ("strength"); break;
        case 2:
          roff_msg_append ("intelligence"); break;
        case 3:
          roff_msg_append ("wisdom"); break;
        case 4:
          roff_msg_append ("dexterity"); break;
        case 5:
          roff_msg_append ("constitution"); break;
        case 6:
          roff_msg_append ("charisma"); break;
        default:
          break;
        }
      roff_msg_append (" is magically sustained from being lowered.  (HA) stands for Holy Avenger.");
      break;
    case SN_DF:
      roff_msg_append (" is a very fine magical weapon that makes the wielder able to fall from great heights without any damage, heal faster, and see invisible creatures.  It also confers the ability of stealth, so that enemies are less likely to see the wielder by ");
      roff_msg_points (i_ptr->p1);
      roff_msg_append (".");
      roff_msg_append ("  It provides resistance to fire, cold, lightning and acid attacks.  Only one third of the regular damage is done.");
      roff_msg_append ("  Lastly, the wielder of this weapon cannot be paralyzed, held, slept, or slowed down.  (DF) stands for Defender.");
      break;
    case SN_SA:
      roff_msg_append (" magically does double damage when used against animals.  (SA) stands for Slay Animal.");
      break;
    case SN_SD:
      roff_msg_append (" magically does quadruple damage when used against dragons.  (SD) stands for Slay Dragon.");
      break;
    case SN_SE:
      roff_msg_append (" magically does double damage when used against evil enemies.  (SE) stands for Slay Evil.");
      break;
    case SN_SU:
      roff_msg_append (" magically does triple damage when used against undead enemies.  (SU) stands for Slay Undead.");
      break;
    case SN_FT:
      roff_msg_append (" magically does fifty-percent more damage when used against enemies that have a susceptibility to flame-based attacks.  (FT) stands for Flame-Tongue.");
      break;
    case SN_FB:
      roff_msg_append (" magically does fifty-percent more damage when used against enemies that have a susceptibility to frost-based attacks.  (FB) stands for Frost Brand.");
      break;
    case SN_FREE_ACTION:
      roff_msg_append (" magically makes the wearer immune to being paralyzed, being held, being slowed down, or being slept.");
      break;
    case SN_SLAYING:
      roff_msg_append (" magically makes it easier to hit and damage enemies.");
      break;
    case SN_CLUMSINESS:
      roff_msg_append (" magically curses the wearer so that they are less dextrous by ");
      roff_msg_points (abs (i_ptr->p1));
      roff_msg_append (".");
      break;
    case SN_WEAKNESS:
      roff_msg_append (" magically curses the wearer so that they have ");
      roff_msg_points (abs (i_ptr->p1));
      roff_msg_append (" less strength.");
      break;
    case SN_SLOW_DESCENT:
      roff_msg_append (" magically makes the wearer able to fall down great distances without getting hurt.");
      break;
    case SN_SPEED:
      roff_msg_printf (" magically makes the move %d times faster.", i_ptr->p1 + 1);
      break;
    case SN_STEALTH:
      roff_msg_append (" magically makes it harder for enemies to notice you.");
      break;
    case SN_SLOWNESS:
      roff_msg_printf (" magically makes the wearer move %d times slower.", abs (i_ptr->p1) + 1);
      break;
    case SN_NOISE:
      roff_msg_append (" magically makes enemies notice the wearer.");
      break;
    case SN_GREAT_MASS:
      roff_msg_append (" are magically cursed to be very heavy.");
      break;
    case SN_INTELLIGENCE:
      roff_msg_append (" magically improves the intelligence of the wearer by ");
      roff_msg_points (i_ptr->p1);
      roff_msg_append (".");
      break;
    case SN_WISDOM:
      roff_msg_printf (" magically improves the wisdom of the wearer by ");
      roff_msg_points (i_ptr->p1);
      roff_msg_append (".");
      break;
    case SN_INFRAVISION:
      roff_msg_append (" magically makes the wearer able to see warm-blooded creatures in darkness from a distance .");
      break;
    case SN_MIGHT:
      roff_msg_append (" magically improves the wearer's constitution, dexterity, and strength all by ");
      roff_msg_points (i_ptr->p1);
      roff_msg_append (".  It also makes the wearer immune to being paralyzed, being held, being slowed down, or being slept.");
      break;
    case SN_LORDLINESS:
      roff_msg_append (" magically improves the wearer's charisma and wisdom by ");
      roff_msg_points (i_ptr->p1);
      break;
    case SN_MAGI:
      roff_msg_append (" magically makes the wearer resistant to lightning, cold, cold, and fire attacks, such that only one-third of the regular damage is done.  It also magically improves the intelligence of the wearer by ");
      roff_msg_points (i_ptr->p1);
      roff_msg_append (".");
      break;
    case SN_BEAUTY:
      roff_msg_append (" magically improves the charisma of the wearer by ");
      roff_msg_points (i_ptr->p1);
      roff_msg_append (".");
      break;
    case SN_SEEING:
      roff_msg_append (" magically makes the wearer able to see invisible creatures.  It also magically improves the wearer's chances at finding secret doors and traps.");
      break;
    case SN_REGENERATION:
      roff_msg_append (" magically makes the wearer heal faster.");
      break;
    case SN_STUPIDITY:
      roff_msg_append (" magically curses the wearer with less intelligence by ");
      roff_msg_points (abs (i_ptr->p1));
      roff_msg_append (".");
      break;
    case SN_DULLNESS:
      roff_msg_append (" magically curses the wearer with less wisdom by ");
      roff_msg_points (abs (i_ptr->p1));
      roff_msg_append (".");
      break;
    case SN_BLINDNESS:
      roff_msg_append (" magically removes sight from the wearer.");
      break;
    case SN_TIMIDNESS:
      roff_msg_append (" magically curses the wearer with a lack of courage.");
      break;
    case SN_TELEPORTATION:
      roff_msg_append (" magically curses the wearer to be periodically transported to a different location.");
      break;
    case SN_UGLINESS:
      roff_msg_append (" magically curses the wearer with less charisma by ");
      roff_msg_points (abs (i_ptr->p1));
      roff_msg_append (".");
      break;
    case SN_PROTECTION:
      roff_msg_append (" magically provides ");
      roff_msg_points (i_ptr->toac);
      roff_msg_append (" of protection to the wearer from enemies.");
      break;
    case SN_IRRITATION:
      roff_msg_append (" magically curses the wearer to be more noticeable to enemies.");
      break;
    case SN_VULNERABILITY:
      roff_msg_append (" magically curses the wearer so that the attacks of their enemies are more effective.");
      break;
    case SN_ENVELOPING:
      roff_msg_append (" magically curses the wearer so that the attacks of their enemies are more effective.  It also makes the wearer less able to hit and damage enemies.");
      break;
    case SN_FIRE:
      roff_msg_append (" magically makes the wielder do an extra fifty-percent damage against enemies that are susceptible to fire.  It also makes it much easier for the wielder to hit and damage enemies.");
      break;
    case SN_SLAY_EVIL:
      roff_msg_append (" magically makes the wielder do double damage against enemies that are evil.  It also makes it much easier for the wielder to hit and damage enemies.");
      break;
    case SN_DRAGON_SLAYING:
      roff_msg_append (" magically makes the wielder do quadruple damage against dragons.  It also makes it much easier for the wielder to hit and damage enemies.");
      break;
    case SN_EMPTY:
      roff_msg_append (" is empty.");
      break;
    case SN_LOCKED:
      roff_msg_append (" is locked.");
      break;
    case SN_POISON_NEEDLE:
      roff_msg_append (" has a poison-needle trap that needs to be disarmed before opening.");
      break;
    case SN_GAS_TRAP:
      roff_msg_append (" has a gas trap that needs to be disarmed before opening.");
      break;
    case SN_EXPLOSION_DEVICE:
      roff_msg_append (" has an explosive charge that needs to be disarmed before opening.");
      break;
    case SN_SUMMONING_RUNES:
      roff_msg_append (" has strange summoning runes carved into it, and it needs to be disarmed before opening.");
      break;
    case SN_MULTIPLE_TRAPS:
      roff_msg_append (" has a collection of traps that need to be disarmed before opening.");
      break;
    case SN_DISARMED:
      roff_msg_append (" has been disarmed, but it is still closed.");
      break;
    case SN_UNLOCKED:
      roff_msg_append (" is unlocked, but still closed.");
      break;
    case SN_SLAY_ANIMAL:
      roff_msg_append (" magically makes the wielder do double damage against animals.  It also makes it much easier for the wielder to hit and damage enemies.");
      break;
    default:
      break;
    }
  if (i_ptr->ident & ID_DAMD)
    roff_msg_append ("  It is damned.");
  free (name);
}
  
void
display_object_inscription (inven_type *i_ptr)
{
  if (i_ptr->inscrip[0] != '\0')
    {
      roff_msg_append (", and ");
      if (i_ptr->number == 1)
        roff_msg_append ("it bears");
      else
        roff_msg_append ("they bear");
      roff_msg_printf (" an inscription that reads '%s'.", i_ptr->inscrip);
    }
}

void
display_armor_abilities (inven_type *i_ptr)
{
  char *name = get_item_kind_name (i_ptr->tval);
  roff_msg_printf ("The %s", name);
  free (name);
  if (!known2_p (i_ptr))
    roff_msg_printf (" has an armor-class rating of %d.", i_ptr->ac);
  else
    {
      if (i_ptr->toac < 0)
        {
          roff_msg_printf (" has an armor-class rating of %d, "
                           "with a negative bonus of ", i_ptr->ac);
          roff_msg_points (i_ptr->toac);
          roff_msg_printf (", for a total of %d.", i_ptr->ac + i_ptr->toac);
        }
      else if (i_ptr->toac > 0)
        {
          roff_msg_printf (" has an armor-class rating of %d, "
                           "with a bonus of +", i_ptr->ac);
          roff_msg_points (i_ptr->toac);
          roff_msg_printf (", for a total of %d.", i_ptr->ac + i_ptr->toac);
        }
      else
        roff_msg_printf (" has an armor-class rating of %d.", i_ptr->ac);
      if (i_ptr->tohit < 0)
        roff_msg_printf ("  Its bulkiness gives it a -%d penalty when trying to hit enemies.", i_ptr->tohit);
      else if (i_ptr->tohit > 0)
        roff_msg_printf ("  It provides a bonus of +%d when trying to hit enemies.", i_ptr->tohit);
    }
}

static void
display_hitdam (inven_type *i_ptr)
{
  if (known2_p (i_ptr))
    {
      if (i_ptr->tohit && i_ptr->todam)
        roff_msg_printf ("  It has a hitting bonus of %c%d, "
                         "and a damage bonus of %c%d.", 
                         i_ptr->tohit < 0 ? '-' : '+', abs (i_ptr->tohit),
                         i_ptr->todam < 0 ? '-' : '+', abs (i_ptr->todam));
      else if (i_ptr->tohit)
        roff_msg_printf ("  It has a hitting bonus of %c%d.",
                         i_ptr->tohit < 0 ? '-' : '+', abs (i_ptr->tohit));
      else if (i_ptr->todam)
        roff_msg_printf ("  It has a damage bonus of %c%d.",
                         i_ptr->todam < 0 ? '-' : '+', abs (i_ptr->todam));
    }
}

void
display_bow_abilities (inven_type *i_ptr)
{
  int t = 0;
  roff_msg_append ("The ");
  if (strstr (object_list[i_ptr->index].name, "Sling"))
    {
      roff_msg_append ("sling");
      t = 1;
    }
  else if (strstr (object_list[i_ptr->index].name, "Crossbow"))
    {
      roff_msg_append ("crossbow");
      t = 3;
    }
  else
    {
      roff_msg_append ("bow");
      t = 2;
    }

  if (i_ptr->p1 == 1 || i_ptr->p1 == 2)
    roff_msg_append (" does double damage of the projectile it fires.");
  else if (i_ptr->p1 == 3 || i_ptr->p1 == 5)
    roff_msg_append (" does triple damage of the projectile it fires.");
  else if (i_ptr->p1 == 4 || i_ptr->p1 == 6)
    roff_msg_append (" does quadruple damage of the projectile it fires.");
  else
    roff_msg_append (" can't hit anything because it's cursed.");
  switch (t)
    {
    case 1:
      roff_msg_append ("  Rounded pebbles and iron shots are the preferred projectiles for this weapon.");
      break;
    case 2:
      roff_msg_append ("  Arrows are the preferred projectiles for this weapon.");
      break;
    case 3:
      roff_msg_append ("  Bolts are the preferred projectiles for this weapon.");
      break;
    default:
      break;
    }
  display_hitdam (i_ptr);
}
    
void
display_weapon_abilities (inven_type *i_ptr)
{
  roff_msg_printf ("When you hit the enemy with this, it does %dd%d damage, which is a minimum of %d and a maximum of %d hit-points.", i_ptr->damage[0], i_ptr->damage[1], i_ptr->damage[0], i_ptr->damage[0] * i_ptr->damage[1]);
  display_hitdam (i_ptr);
}

void
ident_char ()
{
  char command, query;
  register int i, n;
  int tval = 0;
  int stone_wall_flag = 0;
  int misc_armor_flag = 0;
  int arrow_flag = 0;
  int bow_flag = 0;

  roff_msg_start ();
  if (get_com ("Enter character to be identified :", &command))
    switch (command)
      {
        /* every printable character is listed here, in the order in 
         * which they appear in the ASCII character set */
      case ' ':
        roff_msg_append ("  - An open pit.");
        tval = TV_VIS_TRAP;
        break;
      case '!':
        roff_msg_append ("! - A potion.");
        tval = TV_POTION1;
        break;
      case '"':
        roff_msg_append ("\" - An amulet, periapt, or necklace.");
        tval = TV_AMULET;
        break;
      case '#':
        roff_msg_append ("# - A stone wall.");
        stone_wall_flag = 1;
        break;
      case '$':
        roff_msg_append ("$ - Treasure.");
        tval = TV_GOLD;
        break;
      case '%':
        if (highlight_seams == TRUE)
          {
            roff_msg_append ("% - A magma or quartz vein.");
            stone_wall_flag = 1;
          }
        else
          roff_msg_append ("% - Not used.");
        break;
      case '&':
        roff_msg_append ("& - Treasure chest.");
        tval = TV_CHEST;
        break;
      case '\'':
        roff_msg_append ("' - An open door.");
        tval = TV_OPEN_DOOR;
        break;
      case '(':
        roff_msg_append ("( - Soft armor.");
        tval = TV_SOFT_ARMOR;
        break;
      case ')':
        roff_msg_append (") - A shield.");
        tval = TV_SHIELD;
        break;
      case '*':
        roff_msg_append ("* - Gems.");
        tval = TV_GOLD;
        break;
      case '+':
        roff_msg_append ("+ - A closed door.");
        tval = TV_CLOSED_DOOR;
        break;
      case ',':
        roff_msg_append (", - Food or mushroom patch.");
        tval = TV_FOOD;
        break;
      case '-':
        roff_msg_append ("- - A wand");
        tval = TV_WAND;
        break;
      case '.':
        roff_msg_append (". - Floor.");
        break;
      case '/':
        roff_msg_append ("/ - A pole weapon.");
        tval = TV_POLEARM;
        break;
        /* case '0': prt("0 - Not used."); break; */
      case '1':
        if (should_display_extra_assistance ((command - '0') * -1))
          {
            roff_msg_append ("1 - The entrance to the General Store, where adventurers can buy the essentials for exploring the mines of Moria, such as digging tools, food and torches.");
            increment_extra_assistance ((command - '0' * -1));
          }
        else
          roff_msg_append ("1 - The entrance to the General Store.");
        tval = TV_STORE_DOOR;
        break;
      case '2':
        if (should_display_extra_assistance ((command - '0') * -1))
          {
            roff_msg_append ("2 - The entrance to the Armory, where adventurers can buy and sell shields, boots, body armor and more.");
            increment_extra_assistance ((command - '0') * -1);
          }
        else
          roff_msg_append ("2 - The entrance to the Armory.");
        tval = TV_STORE_DOOR;
        break;
      case '3':
        if (should_display_extra_assistance ((command - '0') * -1))
          {
            roff_msg_append ("3 - The entrance to the Weaponsmith, where adventurers can buy and sell swords, polearms, daggers, hafted weapons, as well as bows and crossbows.");
            increment_extra_assistance ((command - '0') * -1);
          }
        else
          roff_msg_append ("3 - The entrance to the Weaponsmith.");
        tval = TV_STORE_DOOR;
        break;
      case '4':
        if (should_display_extra_assistance ((command - '0') * -1))
          {
            roff_msg_append ("4 - The entrance to the Temple, where adventurers can buy and sell the priestly books, potions, scrolls, and sometimes weapons.");
            increment_extra_assistance ((command - '0') * -1);
          }
        else
          roff_msg_append ("4 - The entrance to the Temple.");
        tval = TV_STORE_DOOR;
        break;
      case '5':
        if (should_display_extra_assistance ((command - '0') * -1))
          {
            roff_msg_append ("5 - The entrance to the Alchemy shop, where adventurers can buy and sell all manner of useful scrolls and potions.");
            increment_extra_assistance ((command - '0') * -1);
          }
        else
          roff_msg_append ("5 - The entrance to the Alchemy shop.");
        tval = TV_STORE_DOOR;
        break;
      case '6':
        if (should_display_extra_assistance ((command - '0') * -1))
          {
            roff_msg_append ("6 - The entrance to the Magic-Users store, where adventurers can buy and sell spell-books, wands, staves, and sometimes rings and amulets.");
            increment_extra_assistance ((command - '0') * -1);
          }
        else
          roff_msg_append ("6 - The entrance to the Magic-Users store.");
        tval = TV_STORE_DOOR;
        break;
        /* case '7': prt("7 - Not used."); break; */
        /* case '8': prt("8 - Not used."); break; */
        /* case '9': prt("9 - Not used.");  break; */
      case ':':
        roff_msg_append (": - Rubble.");
        tval = TV_RUBBLE;
        break;
      case ';':
        roff_msg_append ("; - A loose rock.");
        tval = TV_VIS_TRAP;
        break;
      case '<':
        roff_msg_append ("< - An up staircase.");
        tval = TV_UP_STAIR;
        break;
      case '=':
        roff_msg_append ("= - A ring.");
        tval = TV_RING;
        break;
      case '>':
        roff_msg_append ("> - A down staircase.");
        tval = TV_DOWN_STAIR;
        break;
      case '?':
        roff_msg_append ("? - A scroll.");
        tval = TV_SCROLL;
        break;
      case '@':
        roff_msg_append (py.misc.name);
        break;
      case 'A':
        roff_msg_append ("A - Giant Ant Lion.");
        break;
      case 'B':
        roff_msg_append ("B - The Balrog.");
        break;
      case 'C':
        roff_msg_append ("C - Gelatinous Cube.");
        break;
      case 'D':
        roff_msg_append ("D - An Ancient Dragon (Beware).");
        break;
      case 'E':
        roff_msg_append ("E - Elemental.");
        break;
      case 'F':
        roff_msg_append ("F - Giant Fly.");
        break;
      case 'G':
        roff_msg_append ("G - Ghost.");
        break;
      case 'H':
        roff_msg_append ("H - Hobgoblin.");
        break;
        /* case 'I': prt("I - Invisible Stalker."); break; */
      case 'J':
        roff_msg_append ("J - Jelly.");
        break;
      case 'K':
        roff_msg_append ("K - Killer Beetle.");
        break;
      case 'L':
        roff_msg_append ("L - Lich.");
        break;
      case 'M':
        roff_msg_append ("M - Mummy.");
        break;
        /* case 'N': prt("N - Not used."); break; */
      case 'O':
        roff_msg_append ("O - Ooze.");
        break;
      case 'P':
        roff_msg_append ("P - Giant humanoid.");
        break;
      case 'Q':
        roff_msg_append ("Q - Quylthulg (Pulsing Flesh Mound).");
        break;
      case 'R':
        roff_msg_append ("R - Reptile.");
        break;
      case 'S':
        roff_msg_append ("S - Giant Scorpion.");
        break;
      case 'T':
        roff_msg_append ("T - Troll.");
        break;
      case 'U':
        roff_msg_append ("U - Umber Hulk.");
        break;
      case 'V':
        roff_msg_append ("V - Vampire.");
        break;
      case 'W':
        roff_msg_append ("W - Wight or Wraith.");
        break;
      case 'X':
        roff_msg_append ("X - Xorn.");
        break;
      case 'Y':
        roff_msg_append ("Y - Yeti.");
        break;
        /* case 'Z': prt("Z - Not used."); break; */
      case '[':
        roff_msg_append ("[ - Hard armor.");
        tval = TV_HARD_ARMOR;
        break;
      case '\\':
        roff_msg_append ("\\ - A hafted weapon.");
        tval = TV_HAFTED;
        break;
      case ']':
        roff_msg_append ("] - Miscellaneous armor, including boots, helms, caps, and gloves.");
        misc_armor_flag = 1;
        break;
      case '^':
        roff_msg_append ("^ - A trap.");
        tval = TV_VIS_TRAP;
        break;
      case '_':
        roff_msg_append ("_ - A staff.");
        tval = TV_STAFF;
        break;
        /* case '`': prt("` - Not used."); break; */
      case 'a':
        roff_msg_append ("a - Giant Ant.");
        break;
      case 'b':
        roff_msg_append ("b - Giant Bat.");
        break;
      case 'c':
        roff_msg_append ("c - Giant Centipede.");
        break;
      case 'd':
        roff_msg_append ("d - Dragon.");
        break;
      case 'e':
        roff_msg_append ("e - Floating Eye.");
        break;
      case 'f':
        roff_msg_append ("f - Giant Frog.");
        break;
      case 'g':
        roff_msg_append ("g - Golem.");
        break;
      case 'h':
        roff_msg_append ("h - Harpy.");
        break;
      case 'i':
        roff_msg_append ("i - Icky Thing.");
        break;
      case 'j':
        roff_msg_append ("j - Jackal.");
        break;
      case 'k':
        roff_msg_append ("k - Kobold.");
        break;
      case 'l':
        roff_msg_append ("l - Giant Louse.");
        break;
      case 'm':
        roff_msg_append ("m - Mold.");
        break;
      case 'n':
        roff_msg_append ("n - Naga.");
        break;
      case 'o':
        roff_msg_append ("o - Orc or Ogre.");
        break;
      case 'p':
        roff_msg_append ("p - Person (Humanoid).");
        break;
      case 'q':
        roff_msg_append ("q - Quasit.");
        break;
      case 'r':
        roff_msg_append ("r - Rodent.");
        break;
      case 's':
        roff_msg_append ("s - Skeleton.");
        break;
      case 't':
        roff_msg_append ("t - Giant Tick.");
        break;
        /* case 'u': prt("u - Not used."); break; */
        /* case 'v': prt("v - Not used."); break; */
      case 'w':
        roff_msg_append ("w - Worm or Worm Mass.");
        break;
        /* case 'x': prt("x - Not used."); break; */
      case 'y':
        roff_msg_append ("y - Yeek.");
        break;
      case 'z':
        roff_msg_append ("z - Zombie.");
        break;
      case '{':
        roff_msg_append ("{ - Arrow, bolt, or bullet.");
        arrow_flag = 1;
        break;
      case '|':
        roff_msg_append ("| - A sword or dagger.");
        tval = TV_SWORD;
        break;
      case '}':
        roff_msg_append ("} - Bow, crossbow, or sling.");
        bow_flag = 1;
        break;
      case '~':
        roff_msg_append ("~ - Miscellaneous item.");
        tval = TV_MISC;
        break;
      default:
        roff_msg_append ("Not Used.");
        break;
      }

  if (display_extra_assistance)
    {
      if (tval)
        {
          char *explainer = get_extra_assistance (tval);
          if (explainer)
            {
              roff_msg_append ("  ");
              roff_msg_append (explainer);
              free (explainer);
            }
        }
      else if (stone_wall_flag)
        {
          if (should_display_extra_assistance (DIGGING_ASSIST))
            {
              if  (rogue_like_commands)
                roff_msg_append ("  You can dig into rock by (w)ielding a digging tool, and then hold down CTRL while simultaneously pressing a direction key.  Mining precious metals is a valuable source of income for new adventurers.");
              else
                roff_msg_append ("  You can dig into rock by (w)ielding a digging tool, and then (T)unneling in a given direction.  Mining precious metals is a valuable source of income for new adventurers.");
              increment_extra_assistance (DIGGING_ASSIST);
            }
        }
      else if (misc_armor_flag)
        {
          int show = FALSE;
          if (should_display_extra_assistance (TV_BOOTS))
            {
              increment_extra_assistance (TV_BOOTS);
              show = TRUE;
            }
          if (should_display_extra_assistance (TV_HELM))
            {
              increment_extra_assistance (TV_HELM);
              show = TRUE;
            }
          if (should_display_extra_assistance (TV_GLOVES))
            {
              increment_extra_assistance (TV_GLOVES);
              show = TRUE;
            }
          if (show)
            roff_msg_append ("  You can (w)ear this.");
        }
      else if (arrow_flag)
        {
          int show = FALSE;
          if (should_display_extra_assistance (TV_SLING_AMMO))
            {
              increment_extra_assistance (TV_SLING_AMMO);
              show = TRUE;
            }
          if (should_display_extra_assistance (TV_ARROW))
            {
              increment_extra_assistance (TV_ARROW);
              show = TRUE;
            }
          if (should_display_extra_assistance (TV_BOLT))
            {
              increment_extra_assistance (TV_BOLT);
              show = TRUE;
            }
          if (show)
            {
              if (rogue_like_commands)
                roff_msg_append ("  It is for (t)hrowing at enemies while (w)ielding a bow, crossbow, or sling.");
              else
                roff_msg_append (  "It is for (f)iring at enemies while (w)ielding a bow, crossbow, or sling.");
            }
        }
      else if (bow_flag)
        {
          if (should_display_extra_assistance (TV_BOW))
            {
              if (rogue_like_commands)
                roff_msg_append ("  It is for (t)hrowing arrows (when it's a bow), bolts (when it's a crossbow) or stones (when it's a sling) at your enemies after you've already (w)ielded it.");
              else
                roff_msg_append ("  It is for (f)iring arrows (when it's a bow), bolts (when it's a crossbow) or stones (when it's a sling) at your enemies after you've already (w)ielded it.");
              increment_extra_assistance (TV_BOW);
            }
        }
    }

  /* Allow access to monster memory. -CJS- */
  n = 0;
  for (i = MAX_CREATURES - 1; i >= 0; i--)
    if ((c_list[i].cchar == command) && bool_roff_recall (i))
      {
        if (n == 0)
          {
            roff_msg_append ("  You recall those details? [y/n]\n");
            query = roff_msg_end (TRUE, 0, 0);
            if (query != 'y' && query != 'Y')
              {
                n++;
                break;
              }
            erase_line (0, 0);
            save_screen ();
          }
        n++;
        query = roff_recall (i);
        restore_screen ();
        if (query == ESCAPE)
          break;
      }
  if (n == 0)
    {
      roff_msg_end (-1, 0, 0);
    }
}
